(*
 JungTaek Kim

 Copyright(c) 2000-2004 KAIST/SNU Research On Program Analysis System
 (National Creative Research Initiative Center 1998-2003)
 http://ropas.snu.ac.kr/n

 All rights reserved. This file is distributed under the terms of
 an Open Source License.
*)

signature SyntaxErrSig =
sig

type duplicate_type =
   DupTypeId | DupConId | DupVarId | DupLabel
 | DupStrId | DupFctId | DupSigId

type invalid_type =
   InvTypeId | InvTypeVar | InvConId | InvVarId | InvLabel
 | InvStrId | InvFctId | InvSigId
 | InvType | InvPat | InvAsPat | InvOrPat | InvExp | InvArrUpdate
 | InvDec | InvStrDec | InvRecBind | InvStrExp | InvSigExp
 | InvSpec | InvTopDec | InvTopDir

type msg_style = EngGentle | HanGentle | HanTough

type unmatched_type = Unclosed | DiffFunId

type error_kinds =
    Unmatched of unmatched_type * Location.t * string * Location.t * string
  | Duplicate of duplicate_type * Location.t * Location.t  * string
  | Invalid of invalid_type * Location.t
  | Predefined of Location.t * string
  | Missing of Location.t * string
  | Unknown of Location.t
  | Debug of Location.t * string
  (*
  | Orpaterr of Location.t
  *)

exception Error of error_kinds
exception Escape_error

val curmsg_style: msg_style ref
val report_error: error_kinds -> unit
end

structure NsyntaxErr : SyntaxErrSig = 
struct

open Format
open Location

type duplicate_type =
   DupTypeId | DupConId | DupVarId | DupLabel
 | DupStrId | DupFctId | DupSigId

type invalid_type =
   InvTypeId | InvTypeVar | InvConId | InvVarId | InvLabel
 | InvStrId | InvFctId | InvSigId
 | InvType | InvPat | InvAsPat | InvOrPat | InvExp | InvArrUpdate
 | InvDec | InvStrDec | InvRecBind | InvStrExp | InvSigExp
 | InvSpec | InvTopDec | InvTopDir

type msg_style = EngGentle | HanGentle | HanTough

type unmatched_type = Unclosed | DiffFunId

type error_kinds =
    Unmatched of unmatched_type * Location.t * string * Location.t * string
  | Duplicate of duplicate_type * Location.t * Location.t  * string
  | Invalid of invalid_type * Location.t
  | Predefined of Location.t * string
  | Missing of Location.t * string
  | Unknown of Location.t
  | Debug of Location.t * string
  (*
  | Orpaterr of Location.t
  *)

exception Error of error_kinds
exception Escape_error

val curmsg_style = ref HanGentle

fun print_loc x = Location.print Format.std_formatter x

fun duplicate_type_to_string x =
  let val (msg1, msg2, msg3) =
    case x of
      DupTypeId => ("type name", "타입 이름", "타입 이름")
    | DupConId => ("constructor name", "구성자 이름", "구성자 이름")
    | DupVarId => ("variable name", "변수 이름", "변수 이름")
    | DupLabel => ("record label", "레코드 표지", "레코드 표지")
    | DupStrId => ("structure name", "모듈 이름", "모듈 이름")
    | DupFctId => ("functor name", "펑터 이름", "펑터 이름")
    | DupSigId => ("signature name", "시그너처 이름", "시그너처 이름")
  in 
    case !curmsg_style of
      EngGentle => msg1
    | HanGentle => msg2
    | HanTough  => msg3
  end

fun invalid_type_to_string x =
  let val (msg1, msg2, msg3) =
    case x of
      InvTypeId => ("type name", "타입 이름", "타입 이름")
    | InvTypeVar => ("type variable", "타입 변수", "타입 변수")
    | InvConId => ("constructor name", "구성자 이름", "구성자 이름")
    | InvVarId => ("variable name", "변수 이름", "변수 이름")
    | InvLabel => ("record label", "레코드 표지", "레코드 표지")
    | InvStrId => ("structure name", "모듈 이름", "모듈 이름")
    | InvFctId => ("functor name", "펑터 이름", "펑터 이름")
    | InvSigId => ("signature name", "시그너처 이름", "시그너처 이름")
    | InvType => ("type", "타입", "타입")
    | InvAsPat => ("as pattern", "as 패턴", "as 패턴")
    | InvOrPat => ("or pattern", "or 패턴", "or 패턴")
    | InvPat => ("pattern", "패턴", "패턴")
    | InvExp => ("expression", "프로그램 표현", "프로그램 표현")
    | InvArrUpdate => ("array update", "배열 변경", "배열 변경")
    | InvDec => ("declaration", "프로그램 선언", "프로그램 선언")
    | InvStrDec => ("structure declaration", "모듈 선언", "모듈 선언")
    | InvRecBind => ("recursive binding", "재귀적 선언", "재귀적 선언")
    | InvStrExp => ("structure expression", "모듈 표현", "모듈 표현")
    | InvSigExp => ("signature expression", "시그너처 표현", "시그너처 표현")
    | InvSpec => ("specification", "접속방안", "접속방안")
    | InvTopDec => ("toplevel declaration", "최상위 선언", "최상위 선언")
    | InvTopDir => ("toplevel directive", "최상위 명령", "최상위 명령")
  in 
    case !curmsg_style of
      EngGentle => msg1
    | HanGentle => msg2
    | HanTough  => msg3
  end


fun print_head () = 
  (case !curmsg_style of
     EngGentle => print_string "Syntax error: "
   | HanGentle => print_string "문법오류: "
   | HanTough  => print_string "멍청한 실수: ")

fun print_msg (enggentle, hangentle, hantough) = 
  (case !curmsg_style of
     EngGentle => print_string enggentle
   | HanGentle => print_string hangentle
   | HanTough  => print_string hantough)
  
fun report_error(Unmatched(t, first_loc, first, second_loc, second)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations first_loc second_loc)
      then (print_loc first_loc; print_loc second_loc));
      print_head();
      print_msg(second^" is not matched with "^first^".",
        second^"이(가) "^first^"와 맞지 않습니다.",
        second^"랑 "^first^"이(가) 맞지 않는데."))
  | report_error(Duplicate(t, first_loc, second_loc, id)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations first_loc second_loc)
      then (print_loc first_loc; print_loc second_loc));
      print_head();
      let val msg = duplicate_type_to_string t
       in print_msg(msg^" "^id^" are duplicated.",
            "중복된 "^msg^" "^id^"이(가) 사용되었습니다.",
            msg^" "^id^"이(가) 서로 겹치는데.")
      end)
  | report_error(Invalid(t, loc)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations loc loc)
      then print_loc loc);
      print_head();
      let val msg = invalid_type_to_string t
       in print_msg("invalid "^msg^" is used.",
            "잘못된 "^msg^"이(가) 사용되었습니다.",
            msg^"이(가) 이상해.")
      end)
  | report_error(Predefined(loc, id)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations loc loc)
      then print_loc loc);
      print_head();
      print_msg("reserved name "^id^" is used.",
        "미리 정의된 이름 "^id^"이(가) 사용되었습니다.",
        id^"은(는) 이미 찜해두었으니 쓸수 없어."))
  | report_error(Missing(loc, id)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations loc loc)
      then print_loc loc);
      print_head();
      print_msg("'"^id^"' is missing.",
        "'"^id^"'이(가) 빠졌습니다.",
        "'"^id^"'은(는) 이미 찜해두었으니 쓸수 없어."))
  | report_error(Unknown(loc)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations loc loc)
      then print_loc loc);
      print_head();
      print_msg("unknown error.",
        "알수 없는 에러.",
        "모르는 에러인데."))
  | report_error(Debug(loc,s)) =
      ((if not (String.length !Location.input_name = 0)
         || not (Location.highlight_locations loc loc)
      then print_loc loc);
      print_head();
      print_msg("Debug:"^s,
        "Debug:"^s,
        "Debug:"^s))
end
